;processor type,configuration word,include file

			processor		16F84a
         	__config		_XT_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF
			include         "P16F84a.INC"

;**********************variable declarations***********************

		cblock	H'10'	
			rotslow		;counter register
			rotcnten	;counter enable
			rotcheck	;counter register
			direction	;motor direction
			cnt	 	;counter for patterns
			cnttmp		;copy of cnt
			pcnt1		;pause variable no1
			pcnt2		;pause variable no2
			pcnt1sav	;save of pcnt1
			xbeat		;counter to miss a predefined number of beats
			iotof		;for time calculation to deny rotation
			allow		;allow or deny rotation
			xbeatadj	;to miss a predefined number of beats
			pcnt1_copy	;copy of pcnt1
			lght		;for time calculation to deny rotation
			rncnt		;counter to count beats for direction change
			random		;random number to randomly change direction
			val2		;for random number creation
			val3		;for random number creation
			toofast		;variable to check if tempo is too fast
			dcheck		;check 200msec delay to prevent multi-triggering
			delay		;200msec delay to prevent multi-triggering
			w_temp     	;save of W 
			status_temp	;save of STATUS
		endc	

;******************************************************************
	
	org		0x00		;reset vector
	goto	init			
	org		0x04		;interrupt vector
	goto	interrupt

;**********************init****************************************

init		bsf	STATUS,RP0	;BANK select
		movlw	b'10000'	;RA0-RA4 motor control bits
		movwf	TRISA		;save
		movlw	b'11111111'	;PORTB all inputs
		movwf	TRISB		;save
		movlw	b'10000100'	;int falling,internal CLK,prescaler 1:32 TMR0
		movwf	OPTION_REG	;save

;read the first three locations of EEPROM to get settings
;first setting: how many beats have to be missed

ee_read		bcf	STATUS,RP0	;BANK select	
		movlw	0x00		;first memory location
		movwf	EEADR		;write to EEADR
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,RD	;initialize read
		bcf	STATUS,RP0	;BANK select
		movf	EEDATA,W	;move data to W
		movwf	xbeatadj	;save to xbeatadj

;second setting:rotation speed

		bcf	STATUS,RP0	;BANK select
		movlw	0x01		;second memory location
		movwf	EEADR		;write to EEADR
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,RD	;initialize read
		bcf	STATUS,RP0	;BANK select
		movf	EEDATA,W	;move data to W
		movwf	pcnt1		;save to pcnt1

;third setting:rotation lenght

		bcf	STATUS,RP0	;BANK select
		movlw	0x02		;third memory location
		movwf	EEADR		;write to EEADR
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,RD	;initialize read
		bcf	STATUS,RP0	;BANK select
		movf	EEDATA,W	;move data to W
		movwf	lght		;save to lght

;other register initial values			

		movlw	d'1'		;change direction after the first beat
		movwf	random		;save to the register
		movlw	d'161'		;any number should be except zero
		movwf	val2		;save
		movlw	d'54'		;any number should be except zero
		movwf	val3		;save
		movlw	d'1'		;count from 1
		movwf	rncnt		;save
		movlw	d'1'		;count from 1
		movwf	xbeat		;save
		clrf	rotslow
		clrf	rotcheck
		clrf	rotcnten	;disable counter
		clrf	direction	;initial direction
		clrf	dcheck		;allow reaction on beat
		clrf	cnt		;clear pattern counter
		clrf	delay		;clear delay
		clrf	iotof		;clear iotof
		clrf	allow		;allow rotation
		clrf	toofast		
		clrf	TMR0		;clear TMR0
		movlw	b'10110000'	;GIE on,INTE on,T0IE on
		movwf	INTCON		;save	
		goto	main		;jump to main part of program

;the PIC gets the control patterns from here
;**********************pattern source subroutine***************************

pattern		movwf	cnttmp		;save the copy of cnt
		movlw	b'00000100'	;mask code
		andwf	direction,W	;depends on this that patterns 1-4 or 5-8 
		addwf	cnttmp,W	;choose the exact pattern
		addwf	PCL,F		;add to program counter
		retlw	b'1001'		;rotates one direction
		retlw	b'1100'
		retlw	b'0110'
		retlw	b'0011'	
		retlw	b'1001'		;rotates the opposite direction
		retlw	b'0011'
		retlw	b'0110'
		retlw	b'1100'

pattern2	movwf	cnttmp
		bcf	STATUS,C	;clear Carry flag
		rlf	direction,W	;rotate reg to get value 0 or 8
		andlw	b'00001000'	;mask
		addwf	cnttmp,W
		addwf	PCL,F		;add to program counter
		retlw	b'1001'
		retlw	b'1000'
		retlw	b'1100'
		retlw	b'0100'
		retlw	b'0110'
		retlw	b'0010'
		retlw	b'0011'
		retlw	b'0001'
		retlw	b'1001'
		retlw	b'0001'
		retlw	b'0011'
		retlw	b'0010'
		retlw	b'0110'
		retlw	b'0100'
		retlw	b'1100'
		retlw	b'1000'

;pause must be made between steps
;**********************pause subroutine*************************************

pause		movf	pcnt1,W		;how many to decrement
		movwf	pcnt1_copy	;save the copy
		call	outter		;call outter pause
		decfsz	pcnt1_copy,F	;decrement.reached zero?
		goto	$-2		;no,back 2 lines
		return			;yes,return
outter		movlw   d'210'          ;how many to decrement
      		movwf   pcnt2           ;save
inner   	nop			;time adjust
		nop			;time adjust
		decfsz  pcnt2,F         ;decrement.reached zero?
        	goto    $-3           	;no,back to inner
        	return  		;yes,return

;the program calls this subroutine if the rotation is disabled
;**********************deny subroutine**************************************

deny		bsf	rotcnten,0
deny_1		btfss	PORTB,1		;button pushed on RB1?
		call	xbeat_up	;yes,do setting
		btfss	PORTB,2		;button pushed on RB2?
		call	xbeat_down	;yes,do setting
		btfss	PORTB,3		;button pushed on RB3?
		call	spd_up		;yes,do setting
		btfss	PORTB,4		;button pushed on RB4?
		call	spd_down	;yes,do setting
		btfss	PORTB,5		;button pushed on RB5?
		call	lenghten	;yes,do setting
		btfss	PORTB,6		;button pushed on RB6?
		call	shorten		;yes,do setting
		
		btfsc	allow,0		;normal rotation allowed?
		goto	$+7		;yes,jump
		movlw	d'249'		;no.check if 15sec-
		addwf	rotcheck,W	;-elapsed since last beat-
		btfsc	STATUS,C	;-to start slower rotation
		goto	rotslowsub	;elapsed so jump
		btfss	allow,0		;normal rotation allowed?
		goto	deny_1		;no,jump back
		bcf	rotcnten,0	;yes.disable counter
		clrf	rotslow		;clear counter registers
		clrf	rotcheck
		clrf	allow		;clear allow
		return			;return

rotslowsub	bcf	rotcnten,0	;disable counter
		clrf	rotslow		;clear counter registers
		clrf	rotcheck	
		movf	pcnt1,W
		movwf	pcnt1sav
		movlw	d'50'
		movwf	pcnt1
		movf	cnt,W		;move cnt into W
		call	cntslow		;call sub to get the correct cnt value
		addwf	cnt,F		;save new cnt value

rotslowsub_1	movlw	d'8'		;number of max control patterns one direction
		subwf	cnt,W		;check cnt if it reached 8
		btfsc	STATUS,Z	;reached?
		clrf	cnt		;yes,clear it
		call	pause		;call pause2
		movf	cnt,W		;move cnt into W
		call	pattern2	;get control pattern
		movwf	PORTA		;send control pattern to PORTA
		btfsc	allow,0		;normal rotation allowed?
		goto	to_main		;yes,jump
		incf	cnt,F		;no.increment cnt
		
		btfss	PORTB,1		;button pushed on RB1?
		call	xbeat_up	;yes,do setting
		btfss	PORTB,2		;button pushed on RB2?
		call	xbeat_down	;yes,do setting
		btfss	PORTB,3		;button pushed on RB3?
		call	spd_up		;yes,do setting
		btfss	PORTB,4		;button pushed on RB4?
		call	spd_down	;yes,do setting
		btfss	PORTB,5		;button pushed on RB5?
		call	lenghten	;yes,do setting
		btfss	PORTB,6		;button pushed on RB6?
		call	shorten		;yes,do setting
		goto	rotslowsub_1	;jump back

to_main		movf	cnt,W		;move cnt into W
		call	cntmain		;call sub to get the correct cnt value
		movwf	cnt		;save new cnt value
		clrf	allow		
		movf	pcnt1sav,W
		movwf	pcnt1
		return			;return

cntmain		addwf	PCL,F
		retlw	d'0'
		retlw	d'0'
		retlw	d'1'
		retlw	d'1'
		retlw	d'2'
		retlw	d'2'
		retlw	d'3'
		retlw	d'3'

cntslow		addwf	PCL,F
		retlw	d'1'
		retlw	d'2'
		retlw	d'3'
		retlw	d'4'

;setting adjustments start here
;**********************xbeat_up subroutine*********************************

xbeat_up	bcf	INTCON,GIE	;disable all unmasked interrupts
		incf	xbeatadj,F	;increment
		btfsc	xbeatadj,3	;too high?
		call	beat_upperlimit	;yes,limit it	
		call	xbeat_save	;save to EEPROM
		btfss	PORTB,1		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;**********************xbeat_down subroutine*******************************

xbeat_down	bcf	INTCON,GIE	;disable all unmasked interrupts
		clrf	xbeat		;clear xbeat
		decf	xbeatadj,F	;decrement
		btfsc	STATUS,Z	;too low?
		call	beat_lowerlimit	;yes,limit it
		call	xbeat_save	;save to EEPROM
		btfss	PORTB,2		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;**********************spd_up subroutine***********************************

spd_up		bcf	INTCON,GIE	;disable all interrupts
		decf	pcnt1,F		;decrement
		movlw	d'5'		;check pcnt1-
		subwf	pcnt1,W		;-if its too low or not jet
		btfsc	STATUS,Z	;too low?
		call	spd_lowerlimit	;yes,limit it
		call	spd_save	;save to EEPROM
		btfss	PORTB,3		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;**********************spd_down subroutine*********************************

spd_down	bcf	INTCON,GIE	;disable all interrupts
		incf	pcnt1,F		;increment
		btfsc	pcnt1,7		;too high?
		call	spd_upperlimit	;yes,limit it
		call	spd_save	;save to EEPROM
		btfss	PORTB,4		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;**********************lenghten subroutine*********************************

lenghten	bcf	INTCON,GIE	;disable all interrupts
		decf	lght,F		;decrement
		movlw	d'150'		;check lght-
		subwf	lght,W		;-if its too low or not jet
		btfsc	STATUS,Z	;too low?
		call	lght_lowerlimit	;yes,limit it
		call	lght_save	;save to EEPROM
		btfss	PORTB,5		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;**********************shorten subroutine**********************************

shorten		bcf	INTCON,GIE	;disable all interrupts
		incf	lght,F		;increment
		movlw	d'240'		;check lght-
		subwf	lght,W		;-if its too high or not jet
		btfsc	STATUS,Z	;too high?
		call	lght_upperlimit	;yes,limit it		
		call	lght_save	;save to EEPROM
		btfss	PORTB,6		;button still held down?
		goto	$-1		;yes,loop until released
		bsf	INTCON,GIE	;enable all unmasked interrupts
		return			;return

;setting limiting subroutines start here
;**********************beat_upperlimit*************************************

beat_upperlimit	movlw	d'7'		;set maximum value 
		movwf	xbeatadj	;save
		return			;return

;**********************beat_lowerlimit*************************************

beat_lowerlimit	movlw	d'1'		;set minimum value
		movwf	xbeatadj	;save
		return			;return

;***********************spd_lowerlimit*************************************

spd_lowerlimit	movlw	d'6'		;set minimum value
		movwf	pcnt1		;save
		return			;return

;***********************spd_upperlimit*************************************

spd_upperlimit	movlw	d'127'		;set maximum value
		movwf	pcnt1		;save
		return			;return

;***********************lght_upperlimit************************************

lght_lowerlimit	movlw	d'151'		;set minimum value
		movwf	lght		;save
		return			;return

;**********************lght_lowerlimit*************************************

lght_upperlimit	movlw	d'239'		;set maximum value
		movwf	lght		;save
		return			;return

;save to EEPROM
;**********************xbeat_save subroutine*******************************

xbeat_save	bcf	STATUS,RP0	;BANK select
		movlw	0x00		;first memory location
		movwf	EEADR		;write to EEADR
		movf	xbeatadj,W	;move to W
		movwf	EEDATA		;move to EEDATA
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,WREN	;enable write to the EEPROM
		movlw	0x55		;required sequence(4 lines)
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf	EECON1,WR	;initialize write
		btfsc	EECON1,WR	;write complete?
		goto	$-1		;no,go back until complete
		bcf	EECON1,WREN	;disable write to the EEPROM
		bcf	STATUS,RP0	;BANK select
		return			;return

;**********************spd_save subroutine*********************************

spd_save	bcf	STATUS,RP0	;BANK select
		movlw	0x01		;second memory location
		movwf	EEADR		;write to EEADR
		movf	pcnt1,W		;move to W
		movwf	EEDATA		;move to EEDATA
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,WREN	;enable write to the EEPROM
		movlw	0x55		;required sequence(4 lines)
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf	EECON1,WR	;initialize write
		btfsc	EECON1,WR	;write complete?
		goto	$-1		;no,go back until complete
		bcf	EECON1,WREN	;disable write to the EEPROM
		bcf	STATUS,RP0	;BANK select
		return			;RETURN

;**********************lght_save subroutine********************************

lght_save	bcf	STATUS,RP0	;BANK select
		movlw	0x02		;third memory location
		movwf	EEADR		;write to EEADR
		movf	lght,W		;move to W
		movwf	EEDATA		;move to EEDATA
		bsf	STATUS,RP0	;BANK select
		bsf	EECON1,WREN	;enable write to the EEPROM
		movlw	0x55		;required sequence(4 lines)
		movwf	EECON2
		movlw	0xAA
		movwf	EECON2
		bsf	EECON1,WR	;initialize write
		btfsc	EECON1,WR	;write complete?
		goto	$-1		;no,go back until complete
		bcf	EECON1,WREN	;disable write to the EEPROM
		bcf	STATUS,RP0	;BANK select
		return			;return

;here starts the main program
;***********************main program***************************************

main		movlw	d'4'		;number of max control patterns one direction
		subwf	cnt,W		;check cnt if it reached 4
		btfsc	STATUS,Z	;reached?
		clrf	cnt		;yes,clear it
		call	pause		;call pause
		movf	cnt,W		;move cnt to W
		call	pattern		;get control pattern
		movwf	PORTA		;send control pattern to PORTA
		movf	iotof,W		;move iotof to W
		addwf	lght,W		;check iotof if the motor must be stopped or not
		btfsc	STATUS,C	;reached the preset value?
		call	deny		;yes,stop rotation
		incf	cnt,F		;increment cnt
		btfss	PORTB,1		;button pushed on RB1?
		call	xbeat_up	;yes,do setting
		btfss	PORTB,2		;button pushed on RB2?
		call	xbeat_down	;yes,do setting
		btfss	PORTB,3		;button pushed on RB3?
		call	spd_up		;yes,do setting
		btfss	PORTB,4		;button pushed on RB4?
		call	spd_down	;yes,do setting
		btfss	PORTB,5		;button pushed on RB5?
		call	lenghten	;yes,do setting
		btfss	PORTB,6		;button pushed on RB6?
		call	shorten		;yes,do setting
		goto	main		;goto main and rotate

;Interrupt service routine (ISR)
;***********************Interrupt Service Routine**************************

interrupt	movwf   w_temp          ;save of W
		swapf	STATUS,W        ;save of-
		movwf	status_temp     ;-STATUS
		
		btfsc	INTCON,T0IF	;TMR0 owerflowed?
		call	timer_int	;yes,call its subroutine
		btfss	INTCON,INTF	;RB0 interrupt?
		goto	restore		;no,return from interrupt service routine
		movf	dcheck,W	;test dcheck-
		sublw	d'1'		;-if the reaction is allowed or no
		btfsc	STATUS,Z	;reaction on beat allowed?
		call	del_check	;no,but let check if 230msec passed
		movf	dcheck,W	;test once more-
		sublw	d'1'		;-if dcheck has been changed or no
		btfsc	STATUS,Z	;so reaction allowed?
		goto	no_timeout	;no,prevent multi-triggering,leave ISR
		movf	xbeat,W		;move xbeat to W
		subwf	xbeatadj,W	;this tells how many beats have to be missed
		btfsc	STATUS,Z	;enough beats have been missed?
		call	rand		;yes,check rncnt if direction can be changed
		incf	xbeat,F		;increment
		bsf	dcheck,0	;disable reaction on beat
		bcf	STATUS,RP0	;BANK select
		clrf	TMR0		;clear TMR0
		clrf	toofast
		clrf	iotof		;clear iotof
		clrf	delay		;cllear delay
no_timeout	bcf	INTCON,INTF	;clear interrupt flag

restore		swapf   status_temp,W   ;set-
		movwf	STATUS          ;-STATUS pre-interrupt value
		swapf   w_temp,F	;set-
		swapf   w_temp,W        ;-W pre-interrupt value
		retfie                  ;return from interrupt

;here is the random number generated and the direction changed
;***********************rand subroutine************************************

rand		bsf	allow,0		;allow rotation
		clrf	xbeat		;clear xbeat
		movf	toofast,W	;check-
		addlw	d'207'		;-if the tempo is-
		btfss	STATUS,C	;-too fast
		goto	dir_change	;yes,it is.change dir immediately
		movf	rncnt,W		;check rncnt-
		subwf	random,W	;-if it reached random
		btfsc	STATUS,Z	;reached?
		goto	dir_change	;yes,change direction
		incf	rncnt,F		;no,increment rncnt
		return			;return
dir_change	movf	direction,W	;by this operation the 3rd bit of-
	    	xorlw	b'00000100'	;-direction will change between 1 and 0
		movwf	direction	;save new direction
		movlw	d'1'		;set rncnt initial value
		movwf	rncnt		;save
random_gen	movf    random,W	;move random into W
    		addwf   val2,F		;add it to val2 and store into val2
    		movf    val2,W		;move val2 into W
    		addwf   val3,F		;add it to val3 and save it into val3
    		movf    val3,W		;move val3 into W
    		addwf   random,F	;add it to random
    		swapf   random,F	;swap nibbles of random
		movf	random,W	;move random into W
		andlw	d'7'		;mask out unwanted bits
		movwf	random		;save new random number
		btfsc	STATUS,Z	;random is zero?
		goto	random_gen	;yes,generate another random number
		
		sublw	d'7'		;7-random
		btfsc	STATUS,Z	;random is 7?
		goto	random_gen	;yes,generate another random number
		movf	random,W	;movw random to W
		sublw	d'6'		;6-random
		btfsc	STATUS,Z	;random is 6?
		goto	random_gen	;yes,generate another random number
		movf	random,W	;move random to W
		sublw	d'5'		;5-random
		btfsc	STATUS,Z	;random is 5?
		goto	random_gen	;yes,generate another random number
		return			;no,return

;**********************Timer0 overflow subroutine**************************

timer_int	incf	iotof,F		;increment iotof
		incf	delay,F		;increment delay
		incf	toofast,F
		btfsc	rotcnten,0	;counter enabled?
		call	rotcntstart	;yes,call sub
		bcf	INTCON,T0IF	;clear interrupt flag
		return			;return

rotcntstart	incf	rotslow,F
		btfsc	STATUS,Z	;rotslow overflowed?
		incf	rotcheck,F	;yes,increment rotcheck
		return	

;*********************delay check subroutine*******************************

del_check	movf	delay,W		;check delay if-
		addlw	d'231'		;-200msec alredy passed
		btfss	STATUS,C	;passed?
		return			;no,return
		clrf	dcheck		;yes,allow reaction on beat
		return			;return

;******end of music controlled stepper motor for discolight effects*******
		
		end	
